@@ -1,5 +1,6 @@
 """Base models for "Content", including the indexing and search features
 that we want any piece of content to have."""
+
 import uuid
 
 from django.conf import settings
@@ -10,7 +11,6 @@ from django.db.models import Model
 from django.template.defaultfilters import slugify
 from django.utils import timezone
 from django.utils.html import strip_tags
-from bulbs.content import TagCache
 import elasticsearch
 from elasticutils import F
 from elastimorphic.base import (
@@ -19,36 +19,58 @@ from elastimorphic.base import (
     SearchManager,
 )
 from polymorphic import PolymorphicModel
-from .shallow import ShallowContentS, ShallowContentResult
 from djbetty import ImageField
 
+from bulbs.content import TagCache
+from .shallow import ShallowContentS, ShallowContentResult
+
 try:
     from bulbs.content.tasks import index as index_task  # noqa
     from bulbs.content.tasks import update as update_task  # noqa
     CELERY_ENABLED = True
 except ImportError:
+    index_task = lambda *x: x
+    update_task = lambda *x: x
     CELERY_ENABLED = False
 
 
 class Tag(PolymorphicIndexable, PolymorphicModel):
-    """Model for tagging up Content."""
+    """Model for tagging up Content.
+    """
+
     name = models.CharField(max_length=255)
     slug = models.SlugField(unique=True)
-
+    # additional manager to handle querying with ElasticSearch
     search_objects = SearchManager()
 
     def __unicode__(self):
+        """unicode friendly name
+        """
         return '%s: %s' % (self.__class__.__name__, self.name)
 
     def save(self, *args, **kwargs):
+        """sets the `slug` values as the name
+
+        :param args: inline arguments (optional)
+        :param kwargs: keyword arguments (optional)
+        :return: `super.save()`
+        """
         self.slug = slugify(self.name)
         return super(Tag, self).save(*args, **kwargs)
 
     def count(self):
+        """gets the count of instances saved in the cache
+
+        :return: `int`
+        """
         return TagCache.count(self.slug)
 
     @classmethod
     def get_mapping_properties(cls):
+        """provides mapping information for ElasticSearch
+
+        :return: `dict`
+        """
         props = super(Tag, cls).get_mapping_properties()
         props.update({
             "name": {"type": "string", "analyzer": "autocomplete"},
@@ -58,6 +80,10 @@ class Tag(PolymorphicIndexable, PolymorphicModel):
         return props
 
     def extract_document(self):
+        """instantiates a dict representation of the object from ElasticSearch
+
+        :return: `dict`
+        """
         data = super(Tag, self).extract_document()
         data.update({
             "name": self.name,
@@ -68,28 +94,48 @@ class Tag(PolymorphicIndexable, PolymorphicModel):
 
     @classmethod
     def get_serializer_class(cls):
+        """gets the serializer class for the model
+
+        :return: `rest_framework.serializers.Serializer`
+        """
         from .serializers import TagSerializer
         return TagSerializer
 
 
 class FeatureType(models.Model):
+    """
+    special model for featured content
+    """
+
     name = models.CharField(max_length=255)
     slug = models.SlugField(unique=True)
 
     def __unicode__(self):
+        """unicode friendly name
+        """
         return self.name
 
     def save(self, *args, **kwargs):
+        """sets the `slug` values as the name
+
+        :param args: inline arguments (optional)
+        :param kwargs: keyword arguments (optional)
+        :return: `super.save()`
+        """
         if self.slug is None or self.slug == "":
             self.slug = slugify(self.name)
         return super(FeatureType, self).save(*args, **kwargs)
 
 
 class ContentManager(SearchManager):
+    """
+    a specialized version of `elastimorphic.base.SearchManager` for `bulbs.content.Content`
+    """
 
     def s(self):
         """Returns a ShallowContentS() instance, using an ES URL from the settings, and an index
-        from this manager's model"""
+        from this manager's model
+        """
 
         base_polymorphic_class = self.model.get_base_class()
         type_ = type(
@@ -103,16 +149,14 @@ class ContentManager(SearchManager):
         """
         Queries using ElasticSearch, returning an elasticutils queryset.
 
-        Allowed params:
-
-         * query
-         * tags
-         * types
-         * feature_types
-         * authors
-         * published
+        :param kwargs: keyword arguments (optional)
+         * query : ES Query spec
+         * tags : content tags
+         * types : content types
+         * feature_types : featured types
+         * authors : authors
+         * published : date range
         """
-
         results = self.s()
 
         if "query" in kwargs:
@@ -129,7 +173,8 @@ class ContentManager(SearchManager):
             if "after" in kwargs:
                 results = results.query(published__gte=kwargs["after"], must=True)
         else:
-            if kwargs.get("published", True) and not "status" in kwargs:  # TODO: kill this "published" param. it sucks
+            # TODO: kill this "published" param. it sucks
+            if kwargs.get("published", True) and not "status" in kwargs:
                 now = timezone.now()
                 results = results.query(published__lte=now, must=True)
 
@@ -162,7 +207,8 @@ class ContentManager(SearchManager):
         model_types = self.model.get_mapping_type_names()
         if types:
             results = results.doctypes(*[
-                type_classname for type_classname in types \
+                type_classname for type_classname
+                in types
                 if type_classname in model_types
             ])
         else:
@@ -170,6 +216,11 @@ class ContentManager(SearchManager):
         return results
 
     def in_bulk(self, pks):
+        """performs a GET from ElasticSearch en masse
+
+        :param pks: iterable of object primary keys
+        :return: `list`
+        """
         results = self.es.multi_get(pks, index=self.model.get_index_name())
         ret = []
         for r in results["docs"]:
@@ -181,26 +232,28 @@ class ContentManager(SearchManager):
 
 
 class Content(PolymorphicIndexable, PolymorphicModel):
-    """The base content model from which all other content derives."""
+    """
+    The base content model from which all other content derives.
+    """
+
     published = models.DateTimeField(blank=True, null=True)
     last_modified = models.DateTimeField(auto_now=True, default=timezone.now)
     title = models.CharField(max_length=512)
     slug = models.SlugField(blank=True, default='')
     description = models.TextField(max_length=1024, blank=True, default="")
-
     # field used if thumbnail has been manually overridden
     thumbnail_override = ImageField(null=True, blank=True, editable=False)
-
+    # attribution
     authors = models.ManyToManyField(settings.AUTH_USER_MODEL)
     feature_type = models.ForeignKey(FeatureType, null=True, blank=True)
     subhead = models.CharField(max_length=255, blank=True, default="")
-
+    # tagging
     tags = models.ManyToManyField(Tag, blank=True)
-
-    indexed = models.BooleanField(default=True)  # Should this item be indexed?
-
-    _readonly = False  # Is this a read only model? (i.e. from elasticsearch)
-
+    # Should this item be indexed?
+    indexed = models.BooleanField(default=True)
+    # Is this a read only model? (i.e. from elasticsearch)
+    _readonly = False
+    # custom ES manager
     search_objects = ContentManager()
 
     class Meta:
@@ -211,12 +264,14 @@ class Content(PolymorphicIndexable, PolymorphicModel):
         )
 
     def __unicode__(self):
+        """unicode friendly name
+        """
         return '%s: %s' % (self.__class__.__name__, self.title)
 
     @property
     def thumbnail(self):
-        """Read-only attribute that provides the value of the thumbnail to display."""
-
+        """Read-only attribute that provides the value of the thumbnail to display.
+        """
         # check if there is a valid thumbnail override
         if self.thumbnail_override.id is not None:
             return self.thumbnail_override
@@ -226,14 +281,15 @@ class Content(PolymorphicIndexable, PolymorphicModel):
         if first_image is not None:
             return first_image
 
-        # no override for thumbnail and no non-none image field, just return override, which is a blank image field
+        # no override for thumbnail and no non-none image field, just return override,
+        # which is a blank image field
         return self.thumbnail_override
 
     @property
     def first_image(self):
-        """Ready-only attribute that provides the value of the first non-none image that's not the thumbnail override
-        field."""
-
+        """Ready-only attribute that provides the value of the first non-none image that's
+        not the thumbnail override field.
+        """
         # loop through image fields and grab the first non-none one
         for field in self._meta.fields:
             if isinstance(field, ImageField):
@@ -246,6 +302,10 @@ class Content(PolymorphicIndexable, PolymorphicModel):
         return None
 
     def get_absolute_url(self):
+        """produces a url to link directly to this instance, given the URL config
+
+        :return: `str`
+        """
         try:
             url = reverse("content-detail-view", kwargs={"pk": self.pk, "slug": self.slug})
         except NoReverseMatch:
@@ -253,17 +313,21 @@ class Content(PolymorphicIndexable, PolymorphicModel):
         return url
 
     def get_status(self):
-        """Returns a string representing the status of this item
-
-        By default, this is one of "draft", "scheduled" or "published"."""
+        """Returns a string representing the status of this item.
+        By default, this is one of "draft", "scheduled" or "published".
 
+        :return: `str`
+        """
         if self.published:
             return "final"  # The published time has been set
-
         return "draft"  # No published time has been set
 
     @property
     def is_published(self):
+        """determines if the content is/should be live
+
+        :return: `bool`
+        """
         if self.published:
             now = timezone.now()
             if now >= self.published:
@@ -272,17 +336,38 @@ class Content(PolymorphicIndexable, PolymorphicModel):
 
     @property
     def type(self):
+        """gets the ElasticSearch mapping name
+
+        :return: `str`
+        """
         return self.get_mapping_type_name()
 
     def ordered_tags(self):
+        """gets the related tags
+
+        :return: `list` of `Tag` instances
+        """
         tags = list(self.tags.all())
         return sorted(
-            tags, key=lambda tag: ((type(tag) != Tag) * 100000) + tag.count(), reverse=True)
+            tags,
+            key=lambda tag: ((type(tag) != Tag) * 100000) + tag.count(),
+            reverse=True
+        )
 
     def build_slug(self):
+        """strips tagging from the title
+
+        :return: `str`
+        """
         return strip_tags(self.title)
 
     def save(self, *args, **kwargs):
+        """creates the slug, queues up for indexing and saves the instance
+
+        :param args: inline arguments (optional)
+        :param kwargs: keyword arguments
+        :return: `bulbs.content.Content`
+        """
         if not self.slug:
             self.slug = slugify(self.build_slug())[:self._meta.get_field("slug").max_length]
         if self.indexed is False:
@@ -294,6 +379,10 @@ class Content(PolymorphicIndexable, PolymorphicModel):
     # class methods ##############################
     @classmethod
     def get_mapping_properties(cls):
+        """creates the mapping for ElasticSearch
+
+        :return: `dict`
+        """
         properties = super(Content, cls).get_mapping_properties()
         properties.update({
             "published": {"type": "date"},
@@ -331,20 +420,27 @@ class Content(PolymorphicIndexable, PolymorphicModel):
         return properties
 
     def extract_document(self):
+        """maps data returned from ElasticSearch into a dict for instantiating an object
+
+        :return: `dict`
+        """
         data = super(Content, self).extract_document()
         data.update({
-            "published"        : self.published,
-            "last_modified"    : self.last_modified,
-            "title"            : self.title,
-            "slug"             : self.slug,
-            "description"      : self.description,
-            "thumbnail"        : self.thumbnail.id if self.thumbnail else None,
-            "authors": [{
-                "first_name": author.first_name,
-                "id"        : author.id,
-                "last_name" : author.last_name,
-                "username"  : author.username
-            } for author in self.authors.all()],
+            "published": self.published,
+            "last_modified": self.last_modified,
+            "title": self.title,
+            "slug": self.slug,
+            "description": self.description,
+            "thumbnail": self.thumbnail.id if self.thumbnail else None,
+            "authors": [
+                {
+                    "first_name": author.first_name,
+                    "id": author.id,
+                    "last_name": author.last_name,
+                    "username": author.username
+                }
+                for author in self.authors.all()
+            ],
             "tags": [tag.extract_document() for tag in self.ordered_tags()],
             "absolute_url": self.get_absolute_url(),
             "status": self.get_status()
@@ -358,12 +454,26 @@ class Content(PolymorphicIndexable, PolymorphicModel):
 
     @classmethod
     def get_serializer_class(cls):
+        """gets the serializer for the class
+
+        :return: `rest_framework.serializers.Serializer`
+        """
         from .serializers import ContentSerializer
         return ContentSerializer
 
 
 class LogEntryManager(models.Manager):
+    """
+    provides additional manager methods for `bulbs.content.LogEntry` model
+    """
+
     def log(self, user, content, message):
+        """creates a new log record
+
+        :param user: user
+        :param content: content instance
+        :param message: change information
+        """
         return self.create(
             user=user,
             content_type=ContentType.objects.get_for_model(content),
@@ -373,12 +483,17 @@ class LogEntryManager(models.Manager):
 
 
 class LogEntry(models.Model):
+    """
+    log entries for changes to content
+    """
+
     action_time = models.DateTimeField("action time", auto_now=True)
     content_type = models.ForeignKey(ContentType, blank=True, null=True, related_name="change_logs")
-    object_id = models.TextField(("object id"), blank=True, null=True)
-    user = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True, related_name="change_logs")
+    object_id = models.TextField("object id", blank=True, null=True)
+    user = models.ForeignKey(
+        settings.AUTH_USER_MODEL, null=True, blank=True, related_name="change_logs")
     change_message = models.TextField("change message", blank=True)
-
+    # custom manager
     objects = LogEntryManager()
 
     class Meta:
@@ -386,24 +501,33 @@ class LogEntry(models.Model):
 
 
 class ObfuscatedUrlInfo(Model):
-    """Stores info used for obfuscated urls of unpublished content."""
+    """
+    Stores info used for obfuscated urls of unpublished content.
+    """
 
     content = models.ForeignKey(Content)
     create_date = models.DateTimeField()
     expire_date = models.DateTimeField()
     url_uuid = models.CharField(max_length=32, unique=True, editable=False)
 
-    def save(self, force_insert=False, force_update=False, using=None,
-             update_fields=None):
+    def save(self, *args, **kwargs):
+        """sets uuid for url
 
-        if not self.id:
-            # this is a totally new instance, create uuid value
+        :param args: inline arguments (optional)
+        :param kwargs: keyword arguments (optional)
+        :return: `super.save()`
+        """
+        if not self.id:  # this is a totally new instance, create uuid value
             self.url_uuid = str(uuid.uuid4()).replace("-", "")
+        super(ObfuscatedUrlInfo, self).save(*args, **kwargs)
 
-        super(ObfuscatedUrlInfo, self).save()
 
+##
+# signal functions
 
 def content_deleted(sender, instance=None, **kwargs):
+    """removes content from the ES index when deleted from DB
+    """
     if getattr(instance, "_index", True):
         index = instance.get_index_name()
         klass = instance.get_real_instance_class()
@@ -412,4 +536,8 @@ def content_deleted(sender, instance=None, **kwargs):
         except elasticsearch.exceptions.NotFoundError:
             pass
 
+
+##
+# signal hooks
+
 models.signals.pre_delete.connect(content_deleted, Content)
